# db/session.py

from sqlalchemy.orm import sessionmaker
from therix.db.db_manager import DatabaseManager

# Initialize the DatabaseManager singleton
db_manager = DatabaseManager()

# Construct the SQLAlchemy database URL

def get_sql_alchemy_url():
    SQLALCHEMY_DATABASE_URL = db_manager._construct_db_url()
    return SQLALCHEMY_DATABASE_URL
# Get the engine from the DatabaseManager
engine = db_manager._engine

# Optionally dispose the engine if needed
# engine.dispose()

# Create the session factory
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)


from importlib import metadata
from therix.core.agent import Agent
from therix.core.inference_models import GroqLlama370b
from therix.core.system_prompt_config import SystemPromptConfig
from therix.core.output_parser import OutputParserWrapper

GROQ_API_KEY = ''

metadata = {
    "name": "Abhishek Dubey",
}

agent = Agent(name="DEFAULT Agent")
(
    agent.add(GroqLlama370b(config={"groq_api_key": GROQ_API_KEY}))
    .add(SystemPromptConfig(config={"system_prompt": "new-prompt"}))
    .save()
)
print(agent.id)


ans = agent.invoke(question="What is the difference between eating an apple and eating a cake?")
print(ans)



from re import I
from re import I
from click import prompt
from langchain.prompts.chat import ChatPromptTemplate
from langchain_community.vectorstores.pgvector import PGVector
from therix.core import inference_models, pipeline
from therix.utils.rag import get_inference_model
from langfuse.callback import CallbackHandler
from langchain_core.pydantic_v1 import BaseModel, Field
from typing import Any, List
from langchain_core.pydantic_v1 import BaseModel, Field
from typing import List
from therix.services.trace_handler import get_trace_handler

DEFAULT_PROMPT = """
        You are a skilled professional who understands All kinds of documents. 
        You will be provided with Content and keywords.
        You have Analyze given data.
        {{content}}
        {{keywords}}
        Response should be in valid JSON format.
        {{format_instructions}}

"""

BASIC_INSTRUCTIONS = """
If a question seems confusing, simply rephrase it using simpler words. Examples can help too.
When unsure of the user's intent, rephrase the question and ask for confirmation.
Only answer questions related to your knowledge base.
Avoid asking the user questions unrelated to their current request.
Strive for factual answers. If unsure, acknowledge it and offer to find more information.
Always be polite and inclusive in your communication.
Be open to improvement based on user interactions and feedback.
"""


async def async_keyword_search(keyword_search_dict,session_id=None):
        
        retriever = keyword_search_dict["retriever"]
        inference_model = keyword_search_dict["inference_model"]
         
        therix_trace_handler = get_trace_handler(trace_details=keyword_search_dict.get("trace_details"), trace_api=keyword_search_dict.get("trace_api"), system_prompt=keyword_search_dict.get("system_prompt"), pipeline_id=keyword_search_dict.get("pipeline_id"), session_id=keyword_search_dict.get("session").get("session_id"),metadata=keyword_search_dict.get("metadata"))

        
        chain_callbacks = [therix_trace_handler] if therix_trace_handler else []

        content = ""  
        for keyword in keyword_search_dict["keywords"]:
            results = retriever.get_relevant_documents(keyword)
    
            for res in results:
                content += res.page_content + str(res.metadata) + "\n\n---\n\n"
        content = content.rstrip("\n\n---\n\n")

        prompt = keyword_search_dict["prompt"]

        if not prompt:
            prompt = DEFAULT_PROMPT.format(content, keyword_search_dict["keywords"])

        prompt = ChatPromptTemplate.from_messages(
            [
                ("system", prompt),
                ("human",
                 "{BASIC_INSTRUCTIONS} {content} {keywords}")
            ]
        )

        model = get_inference_model(inference_model.name, inference_model.config)
        
        chain = prompt | model | keyword_search_dict["output_parser"]
        if(keyword_search_dict.get("trace_details")):
            response_text = chain.invoke({"content": content, "keywords": keyword_search_dict["keywords"], "format_instructions": keyword_search_dict["output_parser"].get_format_instructions(), "BASIC_INSTRUCTIONS": BASIC_INSTRUCTIONS}, config={"callbacks": [chain_callbacks]})
        else: 
            response_text = chain.invoke({"content": content, "keywords": keyword_search_dict["keywords"], "format_instructions": keyword_search_dict["output_parser"].get_format_instructions(), "BASIC_INSTRUCTIONS": BASIC_INSTRUCTIONS})
        
        return response_text

def keyword_search(keyword_search_dict):

        retriever = keyword_search_dict["retriever"]
        inference_model = keyword_search_dict["inference_model"]


        therix_trace_handler = get_trace_handler(trace_details=keyword_search_dict.get("trace_details"), trace_api=keyword_search_dict.get("trace_api"), system_prompt=keyword_search_dict.get("system_prompt"), pipeline_id=keyword_search_dict.get("pipeline_id"), session_id=keyword_search_dict.get("session").get("session_id"),metadata=keyword_search_dict.get("metadata"))

        
        chain_callbacks = [therix_trace_handler] if therix_trace_handler else []
        

        content = ""  
        for keyword in keyword_search_dict["keywords"]:
            results = retriever.get_relevant_documents(keyword)
    
            for res in results:
                content += res.page_content + str(res.metadata) + "\n\n---\n\n"
        content = content.rstrip("\n\n---\n\n")

        prompt = keyword_search_dict["prompt"]

        if not prompt:
            prompt = DEFAULT_PROMPT.format(content, keyword_search_dict["keywords"])
            if(keyword_search_dict.get("output_parser")):
                prompt+="{{format_instructions}}"
            
        prompt = ChatPromptTemplate.from_messages(
            [
                ("system", prompt),
                ("human",
                 "{BASIC_INSTRUCTIONS} {content} {keywords}")
            ]
        )

        model = get_inference_model(inference_model.name, inference_model.config)
        
        chain = prompt | model | keyword_search_dict["output_parser"]
        if(keyword_search_dict.get("trace_details") or keyword_search_dict.get("trace_api")):
            response_text = chain.invoke({"content": content, "keywords": keyword_search_dict["keywords"], "format_instructions": keyword_search_dict["output_parser"].get_format_instructions(), "BASIC_INSTRUCTIONS": BASIC_INSTRUCTIONS}, config={"callbacks": [therix_trace_handler]})
        else: 
            response_text = chain.invoke({"content": content, "keywords": keyword_search_dict["keywords"], "format_instructions": keyword_search_dict["output_parser"].get_format_instructions(), "BASIC_INSTRUCTIONS": BASIC_INSTRUCTIONS})
        
        return response_text
    
    
async def cloud_keyword_search(keyword_search_dict):
    inference_model = keyword_search_dict["inference_model"]
    model = get_inference_model(inference_model.name, inference_model.config)
    
    therix_trace_handler = get_trace_handler(trace_details=keyword_search_dict.get("trace_details"), trace_api=keyword_search_dict.get("trace_api"), system_prompt=keyword_search_dict.get("system_prompt"), pipeline_id=keyword_search_dict.get("pipeline_id"), session_id=keyword_search_dict.get("session").get("session_id"),metadata=keyword_search_dict.get("metadata"))    
    chain_callbacks = [therix_trace_handler] if therix_trace_handler else []
    
    content = ""
    
    for keyword in keyword_search_dict["keywords"]:
        if(type(keyword_search_dict.get("store")) is not list ):
            relevant_documents = await keyword_search_dict['store'].get_relevant_documents(keyword, topK=4, agent_id = keyword_search_dict['pipeline_id'])
        else:
            relevant_documents = []
            for store in keyword_search_dict.get("store"):
                relevant_documents.append(await store.get_relevant_documents(keyword, topK=4, agent_id = keyword_search_dict['pipeline_id']))  

        if(type (relevant_documents) is list):
            for document in relevant_documents:
                for doc in document["data"]:
                    content += doc["document"]+"\n"+f"{doc['cmetadata']}"+"\n\n---\n\n"
        else:    
            for doc in relevant_documents["data"]:
                content += doc["document"]+"\n"+f"{doc['cmetadata']}"+"\n\n---\n\n"
            
            
    prompt = keyword_search_dict["prompt"]
    
    if not prompt:
        prompt = DEFAULT_PROMPT.format(content, keyword_search_dict["keywords"])
        if(keyword_search_dict.get("output_parser")):
            prompt+=f"format instructions:{keyword_search_dict['output_parser'].get_format_instructions()}"
        prompt.replace("{{context}}", f"context : {content}")
        prompt.replace("{{keywords}}", f"keywords : {keyword_search_dict['keywords']}")
            
    else:
        if(keyword_search_dict.get("output_parser")):
            prompt = prompt.format(content=content, keywords=keyword_search_dict["keywords"], format_instructions=keyword_search_dict["output_parser"].get_format_instructions())
    
    
    if(keyword_search_dict.get("trace_details") or keyword_search_dict.get("trace_api")):
        response : Any = model.invoke(input=prompt, config={"callbacks": [therix_trace_handler]})
    else:
        response : Any = model.invoke(input=prompt)
        
    return response.content
    
    
    
    
       



from therix.core.data_sources import PDFDataSource
from therix.core.embedding_models import (
    BedrockTitanEmbedding,

)
from therix.core.inference_models import Anthropic_Claude_Opus
from therix.core.agent import Agent
import sys

from therix.core.trace import Trace

agent = Agent(name="My New Published Agent")
(
        agent.add(PDFDataSource(config={"files": ["../../test-data/rat.pdf"]}))
        .add(BedrockTitanEmbedding(config={
                                "bedrock_aws_access_key_id" : "",
                                "bedrock_aws_secret_access_key" :"",
                                "bedrock_aws_session_token" :" ",
                                "bedrock_region_name" : "us-east-1"}))
        .add(Anthropic_Claude_Opus(
            config={"claude_api_key":" "}
        ))

        .add(
            Trace(
                config={
                    "secret_key": "sk-lf-e62aa7ce-c4c8-4c77-ad7d-9d76dfd96db1",
                    "public_key": "pk-lf-282ad728-c1d6-4247-b6cd-8022198591a9",
                    "identifier": "my own agent",
                }
            )
        )
        .save()
    )

agent.preprocess_data()
print(agent.id)
ans = agent.invoke("What are some use cases of RAT?")

print(ans)


import uuid
import psycopg
import logging
from langchain_postgres import PostgresChatMessageHistory
import os
import aiohttp

logger = logging.getLogger(__name__)

class TherixChatMessageHistory(PostgresChatMessageHistory):
    def __init__(
        self,
        session_id,
        pipeline_id,
        engine=None,
        table_name=None,
        api_url=None,
    ):
        self.session_id = session_id
        self.pipeline_id = pipeline_id
        self.engine = engine
        self.table_name = table_name
        self.api_url = api_url

        if self.engine:
            self.sync_connection = psycopg.connect(engine)
            super().__init__(table_name, session_id, sync_connection=self.sync_connection)
        else:
            self.sync_connection = None
            self.therix_api_key = os.getenv('THERIX_API_KEY')

    def add_message(self, message_role, message, pipeline_id, session_id):
        try:
            id = uuid.uuid4()
            cursor = self.sync_connection.cursor()
            insert_query = f"""INSERT INTO {self.table_name} (id, message_role, message, pipeline_id, session_id) VALUES (%s, %s, %s, %s, %s);"""
            cursor.execute(insert_query, (id, message_role, message, pipeline_id, session_id))
            self.sync_connection.commit()
            cursor.close()
        except (Exception, psycopg.DatabaseError) as e:
            logging.error(e)

    async def async_add_message(self, message_role, message, pipeline_id, session_id):
        try:
            payload = {
                "message_role": message_role,
                "message": message,
                "pipeline_id": pipeline_id,
                "session_id": session_id
            }
            headers = {
                "THERIX_API_KEY": self.therix_api_key,
                "Content-Type": "application/json"
            }
            async with aiohttp.ClientSession() as session:
                async with session.post(self.api_url, json=payload, headers=headers) as response:
                    response.raise_for_status()
        except aiohttp.ClientError as e:
            logging.error(e)

    def get_message_history(self, session_id):
        try:
            cursor = self.sync_connection.cursor()
            get_query = f"""SELECT message, message_role FROM {self.table_name} WHERE session_id = %s"""
            cursor.execute(get_query, (session_id,))
            columns = list(cursor.description)
            messages = cursor.fetchall()
            cursor.close()

            results = []
            for row in messages:
                row_dict = {}
                for i, col in enumerate(columns):
                    row_dict[col.name] = row[i]
                results.append(row_dict)
            return results
        except (Exception, psycopg.DatabaseError) as e:
            logging.error(e)

    async def async_get_message_history(self, session_id):
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(self.api_url, params={"session_id": session_id}, headers={"THERIX_API_KEY": self.therix_api_key, "Content-Type": "application/json"}) as response:
                    response.raise_for_status()
                    return await response.json()
        except aiohttp.ClientError as e:
            logging.error(e)
            return None




from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.engine.url import URL
from alembic.config import Config
from alembic import command
import threading
import logging
import os
from dotenv import load_dotenv

from therix.entities.models import Base

# Load environment variables from .env file
load_dotenv()

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class DatabaseManager:
    _instance = None
    _engine = None
    _session_factory = None

    def __new__(cls):
        logger.debug("Checking instance...")
        if cls._instance is None:
            with threading.Lock():
                if cls._instance is None:
                    logger.debug("Creating new instance...")
                    cls._instance = super(DatabaseManager, cls).__new__(cls)

                    if cls._is_api_key_present():
                        logger.info("THERIX_API_KEY is present. Skipping database operations.")
                    elif cls._are_db_env_vars_present():
                        cls._engine = cls._create_engine()
                        cls._session_factory = sessionmaker(
                            bind=cls._engine, autoflush=False, autocommit=False
                        )
                        cls._setup_database()
                    else:
                        pass
        return cls._instance

    @classmethod
    def get_session(cls):
        if cls._session_factory is None:
            raise Exception("DatabaseManager is not initialized properly.")
        return scoped_session(cls._session_factory)

    @classmethod
    def _create_engine(cls):
        db_url = cls._construct_db_url()
        return create_engine(db_url)

    @classmethod
    def _construct_db_url(cls):
        drivername = os.getenv("THERIX_DB_TYPE", "postgresql")
        username = os.getenv("THERIX_DB_USERNAME")
        password = os.getenv("THERIX_DB_PASSWORD")
        host = os.getenv("THERIX_DB_HOST")
        port = os.getenv("THERIX_DB_PORT")
        db_name = os.getenv("THERIX_DB_NAME")
        return URL.create(
            drivername=drivername,
            username=username,
            password=password,
            host=host,
            port=port,
            database=db_name,
        ).render_as_string(hide_password=False)

    @staticmethod
    def _setup_database():
        if DatabaseManager._engine:
            Base.metadata.create_all(DatabaseManager._engine)
            logger.info("Database tables created.")

            alembic_cfg = DatabaseManager._get_alembic_config()
            DatabaseManager._upgrade_database(alembic_cfg)

    @staticmethod
    def _get_alembic_config():
        base_dir = os.path.dirname(
            os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        )
        alembic_config_path = os.path.join(base_dir, "therix/alembic.ini")
        alembic_cfg = Config(alembic_config_path)
        alembic_cfg.set_main_option(
            "script_location", os.path.join(base_dir, "therix/alembic")
        )
        alembic_cfg.set_main_option(
            "sqlalchemy.url", str(DatabaseManager._construct_db_url())
        )
        return alembic_cfg

    @staticmethod
    def _upgrade_database(alembic_cfg):
        try:
            if alembic_cfg:
                # command.upgrade(alembic_cfg, "head")
                # logger.info("Database upgraded successfully.")
                pass
        except Exception as e:
            logger.error(f"An error occurred while upgrading the database: {e}")
            raise

    @staticmethod
    def _is_api_key_present():
        
        return bool(os.getenv("THERIX_API_KEY"))

    @staticmethod
    def _are_db_env_vars_present():
        required_vars = [
            "THERIX_DB_USERNAME",
            "THERIX_DB_PASSWORD",
            "THERIX_DB_HOST",
            "THERIX_DB_PORT",
            "THERIX_DB_NAME"
        ]
        for var in required_vars:
            if not os.getenv(var):
                return False
        return True


from therix.core.pipeline_component import PipelineComponent
from therix.entities.models import ConfigType

class PIIFilterConfig(PipelineComponent):
    def __init__(self, config):
        super().__init__(ConfigType.PII_FILTER, 'PII_FILTER', config)


from therix.core.agent import Agent

#Add  two agents to merge with agent 
        
merger_agent = Agent.merge(['c1600014-bca1-4436-96e9-975cf333fa54' , 'f202d57b-7ede-456b-8aa5-975052a275a8'])    

# set agent_id
# merger_agent.set_primary('f202d57b-7ede-456b-8aa5-975052a275a8')

ans = merger_agent.invoke("Give me summary of both the documents")
    
print(ans)


import json
import time
from uuid import uuid4
from therix.core.data_sources import PDFDataSource
from therix.core.embedding_models import AzureOpenAIEmbedding3SmallEmbeddingModel, OpenAITextAdaEmbeddingModel
from therix.core.inference_models import AzureOpenAIGPT3TurboPreviewInferenceModel, GroqMixtral87bInferenceModel, OpenAIGPT4TurboPreviewInferenceModel
from therix.core.inference_models import AzureOpenAIGPT3TurboPreviewInferenceModel, OpenAIGPT4TurboPreviewInferenceModel
from therix.core.embedding_models import BedrockTitanEmbedding, OpenAITextAdaEmbeddingModel
from therix.core.inference_models import BedrockTextExpressV1, OpenAIGPT4TurboPreviewInferenceModel
from therix.core.pipeline import Pipeline
from therix.core.cache import CacheConfig
import sys

from therix.core.trace import Trace


start_time = time.process_time()



# TODO: Init therix with DB details, and license key



## Usage:
# python main.py ad11128d-d2ec-4f7c-8d87-15c1a5dfe1a9 "how does it help in reasoning?"

# if args has pipeline_id, then load the pipeline
## else create new pipeline
if len(sys.argv) > 1:
    pipeline = Pipeline.from_id(sys.argv[1])
    question = sys.argv[2]
    session_id = None

    if len(sys.argv) < 4:
        pass
    else:
        session_id = sys.argv[3]

    ans = pipeline.invoke(question, session_id)
    print(ans)

    end_time = time.process_time()


    cpu_time = end_time - start_time

    print("CPU time", cpu_time)

else:
    pipeline = Pipeline(name="My New Published Pipeline")
    (pipeline
    .add(PDFDataSource(config={'files': ['../../test-data/rat.pdf']}))
    .add(BedrockTitanEmbedding(config={"bedrock_aws_access_key_id" : "",
                                "bedrock_aws_secret_access_key" : ""        ,
                                "bedrock_aws_session_token" : "",
                                "bedrock_region_name" : "us-east-1"}))
    .add(GroqMixtral87bInferenceModel(config={'groq_api_key': GROQ_API_KEY}))
    .add(Trace(config={
        'secret_key': 'sk-lf-e62aa7ce-c4c8-4c77-ad7d-9d76dfd96db1',
        'public_key': 'pk-lf-282ad728-c1d6-4247-b6cd-8022198591a9',
        'identifier': 'cache_pipeline'
    }))
    .add(CacheConfig(config={}))
    .save())

    pipeline.preprocess_data()
    print(pipeline.id)
    ans = pipeline.invoke("Explain ablation study?")

    print(ans)


    end_time = time.process_time()


    cpu_time = end_time - start_time

    print("CPU time", cpu_time)
    
    


from enum import Enum
import os
class DataSourceMaster:
    TEXT = "TEXT"
    PDF = "PDF"
    WEBSITE = "WEBSITE"
    DOCX = "DOCX"
    CSV = "CSV"
    DATABASE = "DATABASE"
    YOUTUBE="YOUTUBE"

class EmbeddingModelMaster:
    OPENAI_TEXT_ADA = 'text-embedding-ada-002'
    OPENAI_TEXT_EMBEDDING_3_LARGE = 'text-embedding-3-large'
    OPENAI_TEXT_EMBEDDING_3_SMALL = 'text-embedding-3-small'
    AZURE_TEXT_ADA='text-embedding-ada-002'
    AZURE_TEXT_EMBEDDING_3_LARGE = 'text-embedding-3-large'
    AZURE_TEXT_EMBEDDING_3_SMALL = 'text-embedding-3-small'
    BEDROCK_TITAN_EMBEDDING='amazon.titan-embed-text-v1'
    BEDROCK_TITAN_MULTIMODAL_EMBEDDING='amazon.titan-embed-image-v1'
    GEMINI_EMBEDDING='models/embedding-001'

class InferenceModelMaster:
    OPENAI_GPT_4_TURBO_PREVIEW = 'gpt-4-turbo-preview'
    OPENAI_GPT_4 = 'gpt-4'
    OPENAI_GPT_4_O = 'gpt-4o'
    OPENAI_GPT_4_O_MINI = 'gpt-4o-mini'
    OPENAI_GPT_3_5_TURBO = 'gpt-3.5-turbo'
    OPENAI_GPT_3_5_TURBO_INSTRUCT = 'gpt-3.5-turbo-instruct'
    AZURE_GPT_4_TURBO_PREVIEW = 'gpt-4-turbo-preview'
    AZURE_GPT_4 = 'gpt-4'
    AZURE_GPT_4_O = 'gpt-4o'
    AZURE_GPT_3_5_TURBO = 'gpt-3.5-turbo'
    AZURE_GPT_3_5_TURBO_INSTRUCT = 'gpt-3.5-turbo-instruct'
    GROQ_LLM_MIXTRAL_8_7_B='mixtral-8x7b-32768'
    GROQ_LLM_LLAMA3_70B= 'llama3-70b-8192'
    GROQ_LLM_GEMMA7B= 'gemma-7b-it'
    GROQ_LLM_LLAMA3_8B= 'llama3-8b-8192'
    GROQ_LLM_LLAMA3_1_8B= 'llama-3.1-8b-instant'
    GROQ_LLM_LLAMA3_1_70B= 'llama-3.1-70b-versatile'
    GROQ_LLM_LLAMA3_1_405B= 'llama-3.1-405b-reasoning'
    BEDROCK_TEXT_EXPRES_V1='amazon.titan-text-express-v1'
    BEDROCK_TEXT_LITE_G1='amazon.titan-text-lite-v1'
    BEDROCK_TEXT_PREMIER_G1='amazon.titan-text-premier-v1'
    GOOGLE_GEMINI_PRO='gemini-pro'
    GOOGLE_GEMINI_1_5_PRO='gemini-1.5-pro'
    ANTHROPIC_CLAUDE_3_OPUS='claude-3-opus-20240229'
    ANTHROPIC_CLAUDE_2_1='claude-2.1'
    ANTHROPIC_CLAUDE_3_HAIKU='claude-3-haiku-20240307-v1'
    ANTHROPIC_CLAUDE_3_SONNET='claude-3-sonnet-20240307-v1'
    ANTHROPIC_CLAUDE_3_5_SONNET='claude-3-5-sonnet-20240620-v1'

class OutputSourceMaster:
    S3 = "S3"
    LOCAL = "LOCAL"
    DATABASE = "DATABASE"


class ChatMessage(Enum):
    CHAT_SAVED = "Message saved successfully!"
    CHAT_FAILED = "Error saving message"
    FAILED_TO_RETRIEVE = "Error retrieving message history"

class PipelineTypeMaster(Enum):
    RAG = 'RAG'
    SUMMARIZER = 'SUMMARIZER'
    DEFAULT = 'DEFAULT'
    
class API_Endpoint:
    if(os.getenv("ENVIRONMENT") == 'DEV'):
        BASE_URL = 'https://cloud-api.dev.therix.ai/api/sdk/'
    else:
        BASE_URL = 'https://cloud-api.therix.ai/api/sdk/'
    
    EMBEDDING_ENDPOINT = f"{BASE_URL}embedding" 
    CHAT_HISTORY_ENDPOINT = f"{BASE_URL}chat-history/"


from ..entities.models import ConfigType

class PipelineComponent:
    def __init__(self, component_type, name, config):
        if component_type not in ConfigType:
            raise ValueError(f"Invalid component type: {component_type}")
        self.type = ConfigType(component_type)
        self.name = name
        self.config = config
    
    def __str__(self):
        return f"{self.type.name} - {self.name} - {self.config}"

class DataSource(PipelineComponent):
    def __init__(self, name, config):
        super().__init__(ConfigType.INPUT_SOURCE, name, config)

class EmbeddingModel(PipelineComponent):
    def __init__(self, name, config):
        super().__init__(ConfigType.EMBEDDING_MODEL, name, config)

class InferenceModel(PipelineComponent):
    def __init__(self, name, config):
        super().__init__(ConfigType.INFERENCE_MODEL, name, config)

class OutputSource(PipelineComponent):
    def __init__(self, name, config):
        super().__init__(ConfigType.OUTPUT_SOURCE, name, config)

class TraceDetails(PipelineComponent):
    def __init__(self, name, config):
        super().__init__(ConfigType.TRACE_DETAILS, name, config)


import pkg_resources    



def get_current_version():
        library_name = 'therix'
        version = pkg_resources.get_distribution(library_name).version
        return version

async def async_get_current_version():
        library_name = 'therix'
        version = pkg_resources.get_distribution(library_name).version
        return version



from therix.core.data_sources import PDFDataSource
from therix.core.embedding_models import (
    BedrockTitanEmbedding,
    OpenAITextAdaEmbeddingModel,)
from therix.core.inference_models import (
    GroqLlama370b
)
from therix.core.agent import Agent

GROQ_API_KEY=''


agent = Agent(name="RAT  Pirate")
(
        agent.add(PDFDataSource(config={"files": ["../../test-data/rat.pdf"]}))
        .add(BedrockTitanEmbedding(config={"bedrock_aws_access_key_id" : "",
                                "bedrock_aws_secret_access_key" : "",
                                "bedrock_aws_session_token" : "",
                                "bedrock_region_name" : "us-east-1"}))
        .add(GroqLlama370b(config={"groq_api_key": GROQ_API_KEY}))
        .save()
    )


agent.preprocess_data()
print(agent.id)


# ASYNCHRONOUS CALL - EXAMPLE
async def call_agent():
        ans = await agent.async_invoke("What are some use cases of RAT?")
        print(ans)
        return ans

# asyncio.run(call_agent()          Add asyncio dependency to run this


import enum
from sqlalchemy import Column, String, Boolean, DateTime, ForeignKey, JSON
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship
import uuid

from sqlalchemy.orm import DeclarativeBase
import sqlalchemy as sa

from therix.core.constants import PipelineTypeMaster


# Creating a base class
class Base(DeclarativeBase):
    pass


class ConfigType(enum.Enum):
    INPUT_SOURCE = "INPUT_SOURCE"
    EMBEDDING_MODEL = "EMBEDDING_MODEL"
    INFERENCE_MODEL = "INFERENCE_MODEL"
    OUTPUT_SOURCE = "OUTPUT_SOURCE"
    TRACE_DETAILS = "TRACE_DETAILS"
    PII_FILTER = "PII_FILTER"
    SUMMARIZER = "SUMMARIZER"
    CACHE_CONFIG = "CACHE_CONFIG"
    SYSTEM_PROMPT = 'SYSTEM_PROMPT'


class Pipeline(Base):
    __tablename__ = "pipelines"

    id = Column(
        UUID(as_uuid=True),
        primary_key=True,
        default=uuid.uuid4,
        unique=True,
        nullable=False,
    )
    name = Column(String, nullable=False)
    type = Column(
        sa.Enum(PipelineTypeMaster)
    )
    status = Column(String, default="IN_DRAFT")  # Adjust default value as needed
    is_deleted = Column(Boolean, default=False)
    deleted_at = Column(DateTime, nullable=True)
    custom_data = Column(JSON, default=dict)

    configurations = relationship("PipelineConfiguration", back_populates="pipeline")
    chat_history = relationship("ChatHistory", back_populates="pipeline")


class PipelineConfiguration(Base):
    __tablename__ = "pipeline_configurations"

    id = Column(
        UUID(as_uuid=True),
        primary_key=True,
        default=uuid.uuid4,
        unique=True,
        nullable=False,
    )
    pipeline_id = Column(UUID(as_uuid=True), ForeignKey("pipelines.id"))
    config_type = Column(
        sa.Enum(ConfigType, name="configtype", create_constraint=True), nullable=False
    )
    name = Column(String, nullable=False)
    config = Column(JSON, default=dict)
    custom_data = Column(JSON, default=dict)
    pipeline = relationship("Pipeline", back_populates="configurations")


class ChatHistory(Base):
    __tablename__ = "chat_history"

    id = Column(
        UUID(as_uuid=True),
        primary_key=True,
        default=uuid.uuid4,
        unique=True,
        nullable=False,
    )
    message = Column(String, nullable=False)
    message_role=Column(String, nullable=False)
    pipeline_id = Column(UUID(as_uuid=True), ForeignKey("pipelines.id"))
    pipeline = relationship("Pipeline", back_populates="chat_history")
    session_id = Column(UUID(as_uuid=True), default=uuid.uuid4, nullable=False)



"""added new pipeline type DEFAULT

Revision ID: 1997c3ff664e
Revises: 3d84bc32c8ee
Create Date: 2024-05-30 16:26:20.496957

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '1997c3ff664e'
down_revision: Union[str, None] = '3d84bc32c8ee'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    # op.add_column('pipelines', sa.Column('type', sa.Enum('RAG', 'SUMMARIZER', 'DEFAULT', name='pipelinetypemaster'), nullable=True))
    # ### end Alembic commands ###
    pass

def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('pipelines', 'type')
    # ### end Alembic commands ###



from therix.core.data_sources import PDFDataSource
from typing import List
from therix.core.embedding_models import AzureOpenAIEmbedding3LargeEmbeddingModel
from therix.core.inference_models import AzureOpenAIGPT4InferenceModel
from therix.core.output_parser import OutputParserWrapper
from therix.core.agent import Agent
from pydantic import BaseModel, Field


agent = Agent(name="My New keyword-search Agent")
(agent
    .add(PDFDataSource(config={'files': ['../../test-data/rat.pdf']}))
    .add(AzureOpenAIEmbedding3LargeEmbeddingModel(config={"azure_deployment" : "", "azure_api_key" : "" , "azure_endpoint" : "" , "openai_api_version" :  ""}))
    .add(AzureOpenAIGPT4InferenceModel(config={"azure_api_key" : "" , "openai_api_version" :  "", "azure_endpoint" : "" , "azure_deployment" : "" , "temperature" : ""}))
    # .add(Trace(config={
    #                 'secret_key': 'sk-lf-e62aa7ce-c4c8-4c77-ad7d-9d76dfd96db1',
    #                 'public_key': 'pk-lf-282ad728-c1d6-4247-b6cd-8022198591a9',
    #                 'identifier': 'keyword_search_agent'
    #      }))
        .save()
        )

agent.preprocess_data()
print(agent.id)
    # ans = agent.invoke("What are some use cases of RAT?")
class TestDetails(BaseModel):
        name: str = Field(description="Name of the Topic")
        description: str = Field(description="Short description of the Topic")
        citations: str = Field(description="add source of every topic, from where it is generated")
        page: str = Field(description="page number of the topic")


class OutputParserJSON(BaseModel):
    tests: List[TestDetails] = Field(description="Topic")
        
mergerAgent = Agent.merge(['0ab4aba8-71bb-4822-8561-ae1f11623104', agent.id])

# mergerAgent.set_primary(agent.id)


keyword_search_dict = {
        "prompt" : "Analyze the provided report content for all the documents provided and include only the sentences that matches {keywords} that is being provided separately and also add the keywords whose records are not found and add description as not found. Content: {content}. Response should be a list of JSON  format. {format_instructions} Example: {{tests: [{{name: 'name of test', description: 'short description of Test'}}]}}",
        "keywords" : ["gender" , "RAT"],
        "output_parser" : OutputParserWrapper.parse_output(pydantic_object=OutputParserJSON)
    }
    
ans = mergerAgent.invoke(keyword_search_params=keyword_search_dict)
# ans = agent.invoke(keyword_search_params=keyword_search_dict)

print(ans)




import pytest
from unittest.mock import patch, MagicMock
from sqlalchemy.engine.url import URL
from therix.db.db_manager import DatabaseManager


class TestDatabaseManager:
    @patch("therix.db.db_manager.DatabaseManager._create_engine")
    @patch("therix.db.db_manager.DatabaseManager._setup_database")
    def test_singleton_pattern(self, mock_setup, mock_engine):
        # Test to ensure that the DatabaseManager is a singleton
        db_manager1 = DatabaseManager()
        db_manager2 = DatabaseManager()
        assert db_manager1 is db_manager2

    @patch("therix.db.db_manager.create_engine")
    def test_create_engine_called_with_correct_url(self, mock_engine):
        # Environment setup for testing
        with patch.dict(
            "os.environ",
            {
                "THERIX_DB_TYPE": "postgresql",
                "THERIX_DB_USERNAME": "user",
                "THERIX_DB_PASSWORD": "pass",
                "THERIX_DB_HOST": "localhost",
                "THERIX_DB_PORT": "5432",
                "THERIX_DB_NAME": "testdb",
            },
        ):
            DatabaseManager._create_engine()
            mock_engine.assert_called_once_with(
                URL.create(
                    drivername="postgresql",
                    username="user",
                    password="pass",
                    host="localhost",
                    port="5432",
                    database="testdb",
                )
            )

    @patch("therix.db.db_manager.scoped_session")
    def test_get_session_raises_exception_if_not_initialized(self, mock_scoped_session):
        # Resetting class variables to test initialization failure
        DatabaseManager._session_factory = None
        with pytest.raises(Exception) as excinfo:
            DatabaseManager.get_session()
        assert "DatabaseManager is not initialized properly." in str(excinfo.value)



"""added config type for cache config

Revision ID: 739c7981bdd8
Revises: f5fb392e6b0a
Create Date: 2024-04-26 09:14:37.750514

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision: str = "739c7981bdd8"
down_revision: Union[str, None] = "f5fb392e6b0a"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(None, "chat_history", ["id"])
    op.create_unique_constraint(None, "pipeline_configurations", ["id"])
    op.create_unique_constraint(None, "pipelines", ["id"])
    op.execute("ALTER TYPE configtype ADD VALUE 'CACHE_CONFIG'")

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "pipelines", type_="unique")
    op.drop_constraint(None, "pipeline_configurations", type_="unique")
    op.drop_constraint(None, "chat_history", type_="unique")
    op.create_table(
        "langchain_pg_collection",
        sa.Column("name", sa.VARCHAR(), autoincrement=False, nullable=True),
        sa.Column(
            "cmetadata",
            postgresql.JSON(astext_type=sa.Text()),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column("uuid", sa.UUID(), autoincrement=False, nullable=False),
        sa.PrimaryKeyConstraint("uuid", name="langchain_pg_collection_pkey"),
        postgresql_ignore_search_path=False,
    )
    op.create_table(
        "langchain_pg_embedding",
        sa.Column("collection_id", sa.UUID(), autoincrement=False, nullable=True),
        sa.Column("embedding", sa.NullType(), autoincrement=False, nullable=True),
        sa.Column("document", sa.VARCHAR(), autoincrement=False, nullable=True),
        sa.Column(
            "cmetadata",
            postgresql.JSON(astext_type=sa.Text()),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column("custom_id", sa.VARCHAR(), autoincrement=False, nullable=True),
        sa.Column("uuid", sa.UUID(), autoincrement=False, nullable=False),
        sa.ForeignKeyConstraint(
            ["collection_id"],
            ["langchain_pg_collection.uuid"],
            name="langchain_pg_embedding_collection_id_fkey",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("uuid", name="langchain_pg_embedding_pkey"),
    )
    # ### end Alembic commands ###



from therix.core.constants import EmbeddingModelMaster
from therix.core.pipeline_component import EmbeddingModel


class OpenAITextAdaEmbeddingModel(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(name=EmbeddingModelMaster.OPENAI_TEXT_ADA, **kwargs)


class OpenAITextEmbedding3SmallEmbeddingModel(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(
            name=EmbeddingModelMaster.OPENAI_TEXT_EMBEDDING_3_SMALL, **kwargs
        )


class OpenAITextEmbedding3LargeEmbeddingModel(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(
            name=EmbeddingModelMaster.OPENAI_TEXT_EMBEDDING_3_LARGE, **kwargs
        )


class AzureOpenAIEmbedding3SmallEmbeddingModel(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(
            name=EmbeddingModelMaster.AZURE_TEXT_EMBEDDING_3_SMALL, **kwargs
        )


class AzureOpenAITextAdaEmbeddingModel(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(name=EmbeddingModelMaster.AZURE_TEXT_ADA, **kwargs)


class AzureOpenAIEmbedding3LargeEmbeddingModel(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(
            name=EmbeddingModelMaster.AZURE_TEXT_EMBEDDING_3_LARGE, **kwargs
        )


class BedrockTitanEmbedding(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(name=EmbeddingModelMaster.BEDROCK_TITAN_EMBEDDING, **kwargs)


class BedrockTitanMultiModalEmbedding(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(
            name=EmbeddingModelMaster.BEDROCK_TITAN_MULTIMODAL_EMBEDDING, **kwargs
        )


class GeminiAIEmbeddings(EmbeddingModel):
    def __init__(self, **kwargs):
        super().__init__(
            name=EmbeddingModelMaster.GEMINI_EMBEDDING, **kwargs
        )


import os
from therix.services.api_service import ApiService
import re

class Prompt:
    @classmethod
    def get_prompt(cls, prompt_name, variables=None):
        api_service = ApiService(therix_api_key=os.getenv("THERIX_API_KEY"))
        if os.getenv("THERIX_API_KEY") is not None:
            response_data = api_service.get(endpoint="prompts/active", params={"prompt_name": prompt_name})
            prompt_template = response_data['data']['prompt']
            
            # Function to replace only provided variables (case-sensitive)
            def replace_placeholders(template, variables):
                if not variables:
                    return template
                
                # Use regex to match and replace placeholders
                def replacer(match):
                    key = match.group(1).strip()
                    return variables.get(key, match.group(0))
                
                return re.sub(r'\{([\w\s]+)\}', replacer, template)
            
            formatted_prompt = replace_placeholders(prompt_template, variables)
            return formatted_prompt
        else:
            raise EnvironmentError("THERIX_API_KEY is not set")